import java.util.*;
import java.io.*;


/** Klasse zur Verwaltung der Artikel
  * @author Bernhard Fiser 0109815
  * @version 1.0 */
public class ArtikelVerwaltung extends HashMap
{
	private static ArtikelVerwaltung	hmap = null;
	private static HashMap			arten;
	private String				path = "/home/fiser/allgemein/studium/itep/abgabe4/testdaten";

	/** Constructor
	  * legt eine neue Hashmap an, in der die Artikelobjekte
	  * gespeichert werden. Der Constructor ist privat und kann daher nicht
	  * von 'auerhalb' instanziert werden. */
	private ArtikelVerwaltung()
	{
		super();
		arten = new HashMap();
	}

	/** Pseudo-Constructor zum instanzieren eines Objekts.
	  * Die Instanzierung erfolgt nur, falls noch kein Objekt dieser Klasse
	  * existiert, andernfalls wird dieses Objekt einmal erstellt. */
	public synchronized static ArtikelVerwaltung getInstance()
	{
		String		line;
		ReadFile	f;

		/* Falls Artikelverwaltung besteht, diese zurckgeben */
		if (hmap != null)
			return(hmap);

		/* Neue Artikelverwaltung erstellen */
		hmap = new ArtikelVerwaltung();

		/* Artenfile einlesen */
		if (!hmap.readArten())
		{
			System.err.println("*** error: readArten");
			hmap = null;
			arten = null;
			return(null);
		}

		/* Artikelfile einlesen */
		if (!hmap.readArtikel())
		{
			System.err.println("*** error: readArtikel");
			hmap = null;
			arten = null;
			return(null);
		}

		/* Artikelverwaltung zurckgeben */
		return(hmap);
	}

	/** Methode zum lesen der Arten
	  * @return true ... alles OK
	  * @return false .. Syntaxerror
	  */
	public synchronized boolean readArten()
	{
		int		lineno = 0;
		Art		a;
		String		line;
		ReadFile	f;

		/* File ffnen */
		f = new ReadFile(path, "arten.txt");
		/* Zeilenweise lesen */
		while ((line = f.readLine()) != null)
		{
			lineno++;
			/* Neues Artenobjekt erstellen */
			a = new Art();
			/* Zeile parsen */
			if (!a.parseLine(line))
			{
				/* Bei Syntaxerror Fehler ausgeben */
				System.err.println("*** error: syntax error in line " + lineno);
				return(false);
			}
			/* Artenobjekt in ArtenHashmap aufnehmen */
			this.arten.put(a.getID().toLowerCase(), a);
		}
		/* Datei schlieen */
		f.close();
		/* Alles OK -> true zurckgeben */
		return(true);
	}

	/** Methode zum Lesen der Artikel 
	  * @return true ... alles OK
	  * @return false .. Syntaxerror 
	  */
	public synchronized boolean readArtikel()
	{
		int		lineno = 0;
		Artikel		a;
		String		line;
		ReadFile	f;

		/* Datei ffnen */
		f = new ReadFile(path, "artikel.txt");
		/* Zeilenweise lesen */
		while ((line = f.readLine()) != null)
		{
			lineno++;
			/* Neuse Artikelobjekt erstellen */
			a = new Artikel((Art) arten.get("bo"));
			/* Zeile parsen */
			if (!a.parseLine(line, this.arten))
			{
				/* Bei Syntaxerror -> Fehlerausgeben */
				System.err.println("*** error: syntax error in line " + lineno);
				return(false);
			}
			/* Artikel in Artikelverwaltung aufnehmen */
			this.put(a.getID().toLowerCase(), a);
		}
		/* Datei schlieen */
		f.close();
		/* Alles OK -> true zurckgeben */
		return(true);
	}

	/** Methode zum speichern der Artikelveraltung
	  * @return true : falls Speichern erfolgreich
	  * @return false : falls Fehler aufgetreten ist */
	public synchronized boolean save()
	{
		Artikel		a;
		Iterator	i;
		PrintWriter	out;

		/* Falls Hashmap nicht initialisiert, Fehler zurckgeben
		 * (sollte niemals eintreten)*/
		if (hmap == null)
			return(false);

		/* Datei zum Schreiben ffnen */
		try
		{
			out = new PrintWriter(new FileOutputStream(new File(path + "/artikel.txt")));
		}
		/* Bei Fehler -> false zurckgeben */
		catch (IOException e)
		{
			return(false);
		}

		/* ber alle Objekte der Artikelverwaltung iterieren */
		i = this.values().iterator();
		while (i.hasNext())
		{
			/* Artikel aus Artikelverwlatung auslesen ... */
			a = (Artikel) i.next();
			/* ... und in Datei schreiben */
			out.println(a.getArt().getDescription() + ";" + a.getDescription() + ";" + a.getID());
		}
		/* Datei schlieen */
		out.close();

		/* Alles OK -> true zurckgeben */
		return(true);
	}

	/** Arten zurckliefern
	  * @return HashMap */
	public synchronized HashMap getArten()
	{
		return(this.arten);
	}

	/** Hauptprogramm zum Testen der Dateien, da
	  * ein derartiger Test im Servlet nicht (nur umstndlich)
	  * mglich ist. */
	public static void main(String argv[])
	{
		ArtikelVerwaltung	av;

		if ((av = ArtikelVerwaltung.getInstance()) == null)
		{
			System.err.println("*** error: could not get instance");
			System.exit(1);
		}
		System.out.println(av.size() + " Artikel geladen");
	}
}
